#include <cycle_count.h>	//For Basic Cycle Count
#include <cycles.h>		//For Cycle Count With Statistics
#include <time.h> 		//For Cycle Count Using Time
#include <stdio.h> 


/*Function Prototypes*/
void Basic_Cycle_Count(void);
void Cycle_Count_With_Statistics(void);
void Cycle_Count_Using_Time(void);
void Some_Function_Or_Code_To_Measure(void);

/*Globals*/
int array[20000];

extern int main(void) 
{ 
	printf("Cycle Counting Facilities\n------------------------\n");
	
	//Call the Basic Cycle Count
	printf("Basic Cycle Count:\n");
	Basic_Cycle_Count();
	printf("\n------------------------\n");
	
	//Call the Cycle Count With Statistics
	printf("Cycle Count With Statistics:\n");
	Cycle_Count_With_Statistics();
	printf("\n------------------------\n");
	
	//Call the Cycle Count Using Time
	printf("Cycle Count Using time.h:\n");
	Cycle_Count_Using_Time();
	printf("\n------------------------");
} 


/*Basic Cycle Count. This function calcualtes the number of cycles taken to
//execute the function 'Some_Function_Or_Code_To_Measure'.*/
void Basic_Cycle_Count(void)
{
	cycle_t start_count;
	cycle_t final_count;
	START_CYCLE_COUNT(start_count);
	
	Some_Function_Or_Code_To_Measure();

	STOP_CYCLE_COUNT(final_count,start_count);
	PRINT_CYCLES("Number of cycles:",final_count);
}


/*Cycle Count With Statistics. This function calculates the average, minimum 
//and maximum number of clock cycles taken to execute the function 
//'Some_Function_Or_Code_To_Measure' over ten runs*/
void Cycle_Count_With_Statistics(void)
{
	int i;
	cycle_stats_t stats;
	
	CYCLES_INIT(stats);
	for (i = 0; i < 10; i++)
	{
		CYCLES_START(stats);
		
		Some_Function_Or_Code_To_Measure();
		
		CYCLES_STOP(stats);
	}
	CYCLES_PRINT(stats);
	CYCLES_RESET(stats);
}


/*Cycle Count Using Time. This function calculates the execution time of a 
//section of code in seconds.*/
void Cycle_Count_Using_Time(void)
{
	 volatile clock_t clock_start; 
     volatile clock_t clock_stop; 
     double secs; 
     clock_start = clock(); 

     Some_Function_Or_Code_To_Measure(); 

     clock_stop = clock(); 
     secs = ((double) (clock_stop - clock_start)) / CLOCKS_PER_SEC; 
     
     printf("Time taken was %e seconds\n",secs); 
}

/*This function is a simple calculation loop to use up some time so we can
//obtain a cycle count.*/
void Some_Function_Or_Code_To_Measure(void) 
{
    int i;
    for (i=0; i<20000; i++)
    {
        array[i] = i*i;
    }
}
